@@ -1,3 +1,4 @@
+#if !defined(TORCH_STABLE_ONLY) && !defined(TORCH_TARGET_VERSION)
 #pragma once
 
 // Test these using #if AT_MKL_ENABLED(), not #ifdef, so that it's
@@ -7,9 +8,9 @@
 // DO NOT put the macros for CUDA libraries in this file; they belong in cuda/CUDAConfig.h
 
 #define AT_MKLDNN_ENABLED() 1
+#define AT_MKLDNN_ACL_ENABLED() 0
 #define AT_MKL_ENABLED() 1
 #define AT_MKL_SEQUENTIAL() 0
-#define AT_FFTW_ENABLED() 0
 #define AT_POCKETFFT_ENABLED() 0
 #define AT_NNPACK_ENABLED() 1
 #define CAFFE2_STATIC_LINK_CUDA() 0
@@ -17,6 +18,11 @@
 #define AT_BUILD_WITH_LAPACK() 1
 #define AT_PARALLEL_OPENMP 1
 #define AT_PARALLEL_NATIVE 0
-#define AT_PARALLEL_NATIVE_TBB 0
 #define AT_BLAS_F2C() 0
 #define AT_BLAS_USE_CBLAS_DOT() 0
+#define AT_KLEIDIAI_ENABLED() 0
+#define AT_USE_EIGEN_SPARSE() 0
+
+#else
+#error "This file should not be included when either TORCH_STABLE_ONLY or TORCH_TARGET_VERSION is defined."
+#endif  // !defined(TORCH_STABLE_ONLY) && !defined(TORCH_TARGET_VERSION)
