@@ -1,3 +1,4 @@
+#if !defined(TORCH_STABLE_ONLY) && !defined(TORCH_TARGET_VERSION)
 #pragma once
 #include <ATen/core/TensorBase.h>
 
@@ -42,3 +43,7 @@ TORCH_CUDA_CPP_API TensorBase empty_strided_cuda(
 
 
 }  // namespace at::detail
+
+#else
+#error "This file should not be included when either TORCH_STABLE_ONLY or TORCH_TARGET_VERSION is defined."
+#endif  // !defined(TORCH_STABLE_ONLY) && !defined(TORCH_TARGET_VERSION)
