@@ -1,3 +1,4 @@
+#if !defined(TORCH_STABLE_ONLY) && !defined(TORCH_TARGET_VERSION)
 #include <ATen/core/Tensor.h>
 #include <ATen/native/DispatchStub.h>
 
@@ -18,3 +19,7 @@ using fused_adagrad_fn = void (*)(
 DECLARE_DISPATCH(fused_adagrad_fn, fused_adagrad_stub)
 
 } // namespace at::native
+
+#else
+#error "This file should not be included when either TORCH_STABLE_ONLY or TORCH_TARGET_VERSION is defined."
+#endif  // !defined(TORCH_STABLE_ONLY) && !defined(TORCH_TARGET_VERSION)
