@@ -1,3 +1,4 @@
+#if !defined(TORCH_STABLE_ONLY) && !defined(TORCH_TARGET_VERSION)
 #pragma once
 
 #include <c10/core/SymInt.h>
@@ -15,3 +16,7 @@ using DimVector = SmallVector<int64_t, kDimVectorStaticSize>;
 using SymDimVector = SmallVector<c10::SymInt, kDimVectorStaticSize>;
 
 } // namespace c10
+
+#else
+#error "This file should not be included when either TORCH_STABLE_ONLY or TORCH_TARGET_VERSION is defined."
+#endif  // !defined(TORCH_STABLE_ONLY) && !defined(TORCH_TARGET_VERSION)
