@@ -1,3 +1,4 @@
+#if !defined(TORCH_STABLE_ONLY) && !defined(TORCH_TARGET_VERSION)
 #pragma once
 
 #include <c10/xpu/XPUMacros.h>
@@ -205,3 +206,7 @@ struct C10_XPU_API DeviceProp {
 #undef DEFINE_EXP_DEVICE_PROP
 
 } // namespace c10::xpu
+
+#else
+#error "This file should not be included when either TORCH_STABLE_ONLY or TORCH_TARGET_VERSION is defined."
+#endif  // !defined(TORCH_STABLE_ONLY) && !defined(TORCH_TARGET_VERSION)
