@@ -1,3 +1,4 @@
+#if !defined(TORCH_STABLE_ONLY) && !defined(TORCH_TARGET_VERSION)
 #pragma once
 
 #include <ATen/core/Generator.h>
@@ -29,3 +30,7 @@ TORCH_PYTHON_API at::Generator THPGenerator_Unwrap(PyObject* state);
 // Creates a new Python object for a Generator. The Generator must not already
 // have a PyObject* associated with it.
 PyObject* THPGenerator_NewWithVar(PyTypeObject* type, at::Generator gen);
+
+#else
+#error "This file should not be included when either TORCH_STABLE_ONLY or TORCH_TARGET_VERSION is defined."
+#endif  // !defined(TORCH_STABLE_ONLY) && !defined(TORCH_TARGET_VERSION)
