@@ -1,3 +1,4 @@
+#if !defined(TORCH_STABLE_ONLY) && !defined(TORCH_TARGET_VERSION)
 #ifndef THP_STREAM_INC
 #define THP_STREAM_INC
 
@@ -13,6 +14,7 @@ struct THPStream {
   int64_t device_index;
   // Used to switch stream context management, initialized lazily.
   PyObject* context;
+  PyObject* weakreflist;
 };
 extern TORCH_API PyTypeObject* THPStreamClass;
 
@@ -25,3 +27,7 @@ inline bool THPStream_Check(PyObject* obj) {
 TORCH_PYTHON_API PyObject* THPStream_Wrap(const c10::Stream& stream);
 
 #endif // THP_STREAM_INC
+
+#else
+#error "This file should not be included when either TORCH_STABLE_ONLY or TORCH_TARGET_VERSION is defined."
+#endif  // !defined(TORCH_STABLE_ONLY) && !defined(TORCH_TARGET_VERSION)
