@@ -1,3 +1,4 @@
+#if !defined(TORCH_STABLE_ONLY) && !defined(TORCH_TARGET_VERSION)
 #pragma once
 
 #include <torch/expanding_array.h>
@@ -703,7 +704,7 @@ struct TORCH_API NLLLossImpl : public Cloneable<NLLLossImpl> {
   /// The options with which this `Module` was constructed.
   NLLLossOptions options;
 
-  /// A manual rescaling weight given to to each class.
+  /// A manual rescaling weight given to each class.
   Tensor weight;
 };
 
@@ -743,7 +744,7 @@ struct TORCH_API CrossEntropyLossImpl : public Cloneable<CrossEntropyLossImpl> {
   /// The options with which this `Module` was constructed.
   CrossEntropyLossOptions options;
 
-  /// A manual rescaling weight given to to each class.
+  /// A manual rescaling weight given to each class.
   Tensor weight;
 };
 
@@ -801,3 +802,7 @@ struct TORCH_API BCEWithLogitsLossImpl
 TORCH_MODULE(BCEWithLogitsLoss);
 
 } // namespace torch::nn
+
+#else
+#error "This file should not be included when either TORCH_STABLE_ONLY or TORCH_TARGET_VERSION is defined."
+#endif  // !defined(TORCH_STABLE_ONLY) && !defined(TORCH_TARGET_VERSION)
