@@ -1,3 +1,4 @@
+#if !defined(TORCH_STABLE_ONLY) && !defined(TORCH_TARGET_VERSION)
 // Copyright (c) Meta Platforms, Inc. and affiliates.
 //
 // This source code is licensed under the BSD-style license found in the
@@ -32,3 +33,7 @@ inline std::string torch_tensor_device_name(const at::Tensor& ten) {
       x.is_cuda(),                                             \
       #x " must be a CUDA tensor; it is currently on device ", \
       torch_tensor_device_name(x))
+
+#else
+#error "This file should not be included when either TORCH_STABLE_ONLY or TORCH_TARGET_VERSION is defined."
+#endif  // !defined(TORCH_STABLE_ONLY) && !defined(TORCH_TARGET_VERSION)
