@@ -1,3 +1,4 @@
+#if !defined(TORCH_STABLE_ONLY) && !defined(TORCH_TARGET_VERSION)
 #pragma once
 
 #include <torch/csrc/distributed/rpc/message.h>
@@ -20,3 +21,7 @@ class TORCH_API ScriptResp final : public RpcCommandBase {
 };
 
 } // namespace torch::distributed::rpc
+
+#else
+#error "This file should not be included when either TORCH_STABLE_ONLY or TORCH_TARGET_VERSION is defined."
+#endif  // !defined(TORCH_STABLE_ONLY) && !defined(TORCH_TARGET_VERSION)
