@@ -1,3 +1,4 @@
+#if !defined(TORCH_STABLE_ONLY) && !defined(TORCH_TARGET_VERSION)
 #if defined(__APPLE__)
 #pragma once
 
@@ -16,3 +17,7 @@ class TORCH_API AOTIModelContainerRunnerMps : public AOTIModelContainerRunner {
 
 } // namespace torch::inductor
 #endif
+
+#else
+#error "This file should not be included when either TORCH_STABLE_ONLY or TORCH_TARGET_VERSION is defined."
+#endif  // !defined(TORCH_STABLE_ONLY) && !defined(TORCH_TARGET_VERSION)
