@@ -1,3 +1,4 @@
+#if !defined(TORCH_STABLE_ONLY) && !defined(TORCH_TARGET_VERSION)
 #pragma once
 #include <c10/core/ScalarType.h>
 #include <c10/util/irange.h>
@@ -424,3 +425,7 @@ class TORCH_API RegisterizerReplacer : public IRMutator {
 TORCH_API StmtPtr registerize(StmtPtr s);
 
 } // namespace torch::jit::tensorexpr
+
+#else
+#error "This file should not be included when either TORCH_STABLE_ONLY or TORCH_TARGET_VERSION is defined."
+#endif  // !defined(TORCH_STABLE_ONLY) && !defined(TORCH_TARGET_VERSION)
