@@ -7,8 +7,8 @@ and this includes tensor subclasses that implement __torch_dispatch__.
 
 import collections
 import typing
-from collections.abc import Iterable
-from typing import Any, Callable, Optional, TypeVar, Union
+from collections.abc import Callable, Iterable
+from typing import Any, Optional, TypeGuard, TypeVar, Union
 
 import torch
 import torch.utils._pytree as pytree
@@ -17,7 +17,19 @@ from torch._subclasses.fake_tensor import get_plain_tensors
 from torch.types import IntLikeType
 from torch.utils._python_dispatch import is_traceable_wrapper_subclass
 
+from .descriptors import (
+    AOTInput,
+    AOTOutput,
+    DummyAOTInput,
+    SubclassGetAttrAOTInput,
+    SubclassGetAttrAOTOutput,
+    SubclassSizeAOTInput,
+    SubclassSizeAOTOutput,
+    SubclassStrideAOTInput,
+    SubclassStrideAOTOutput,
+)
 from .schemas import (
+    FxValue,
     MutationType,
     PlainTensorMeta,
     SubclassCreationMeta,
@@ -112,8 +124,8 @@ def create_subclass_metadata(
 
     new_start_idx = (
         new_start_idx
-        + count_symints * len(filter_symints(a.size()))
-        + count_symints * len(filter_symints(a.stride()))
+        + count_symints * len(enumerate_filter_symints(a.size()))
+        + count_symints * len(enumerate_filter_symints(a.stride()))
     )
 
     return (
@@ -167,12 +179,12 @@ def create_subclass_meta(
     return infos
 
 
-def filter_symints(lst: Iterable[IntLikeType]):
+def enumerate_filter_symints(lst: Iterable[IntLikeType]) -> list[tuple[int, SymInt]]:
     # Capture all SymInts from the iterable.
-    def symint_check(s: IntLikeType) -> bool:
+    def symint_check(s: IntLikeType) -> TypeGuard[SymInt]:
         return isinstance(s, SymInt) and not s.node.is_nested_int()
 
-    return [s for s in lst if symint_check(s)]
+    return [(i, s) for i, s in enumerate(lst) if symint_check(s)]
 
 
 def compute_symint_placeholders(lst: Iterable[Union[None, int, SymInt]]) -> list[bool]:
@@ -180,6 +192,12 @@ def compute_symint_placeholders(lst: Iterable[Union[None, int, SymInt]]) -> list
     return [s is None for s in lst]
 
 
+# Intended to make it easier to define function that is
+# either (AOTInput -> AOTInput) or (AOTOutput -> AOTOutput)
+# but not the other combos
+AOTDescriptor = TypeVar("AOTDescriptor", AOTInput, AOTOutput)
+
+
 # This function takes in a pytree of arguments and unwraps any tensor
 # subclasses.
 #
@@ -194,33 +212,56 @@ def compute_symint_placeholders(lst: Iterable[Union[None, int, SymInt]]) -> list
 # primals (but not tangents) on entry to the forward. See the runtime version of
 # this function below.
 def unwrap_tensor_subclasses(
-    wrapped_args: list[Union[Tensor, int]],
+    wrapped_args: list[FxValue],
+    wrapped_args_descs: list[AOTDescriptor],
     *,
     append_symints: bool,
-):
-    def flatten_subclass(t: Union[Tensor, int], *, out=None):
+) -> tuple[list[FxValue], list[AOTDescriptor]]:
+    def flatten_subclass(
+        t: FxValue,
+        desc: AOTDescriptor,
+        *,
+        out: tuple[list[FxValue], list[AOTDescriptor]],
+    ):
         # unwrap a subclass into plain tensors and their size/stride if "append_symint"
         # is True
         if not is_traceable_wrapper_subclass(t):
-            out.append(t)
+            out[0].append(t)
+            out[1].append(desc)
             return
 
         attrs, _ = t.__tensor_flatten__()
 
         for attr in attrs:
             inner_tensor = getattr(t, attr)
-            flatten_subclass(inner_tensor, out=out)
+            n_desc: Any = (
+                SubclassGetAttrAOTInput(desc, attr)
+                if isinstance(desc, AOTInput)
+                # pyrefly: ignore [bad-argument-type]
+                else SubclassGetAttrAOTOutput(desc, attr)
+            )
+            flatten_subclass(inner_tensor, n_desc, out=out)
 
         if append_symints:
-            out.extend(filter_symints(t.size()))
-            out.extend(filter_symints(t.stride()))
+            sizes = enumerate_filter_symints(t.size())
+            strides = enumerate_filter_symints(t.stride())
+            out[0].extend(s for _, s in sizes)
+            out[0].extend(s for _, s in strides)
+            if isinstance(desc, AOTInput):
+                out[1].extend(SubclassSizeAOTInput(desc, i) for i, _ in sizes)  # type: ignore[misc]
+                out[1].extend(SubclassStrideAOTInput(desc, i) for i, _ in strides)  # type: ignore[misc]
+            else:
+                out[1].extend(SubclassSizeAOTOutput(desc, i) for i, _ in sizes)  # type: ignore[misc]
+                out[1].extend(SubclassStrideAOTOutput(desc, i) for i, _ in strides)  # type: ignore[misc]
 
-    xs_inner: list[Union[int, Tensor, SymInt]] = []
+    xs_inner: list[FxValue] = []
+    descs_inner: list[AOTDescriptor] = []
 
-    for x in wrapped_args:
-        flatten_subclass(typing.cast(Tensor, x), out=xs_inner)
+    for x, desc in zip(wrapped_args, wrapped_args_descs):
+        # pyrefly: ignore [bad-argument-type]
+        flatten_subclass(typing.cast(Tensor, x), desc, out=(xs_inner, descs_inner))
 
-    return xs_inner
+    return xs_inner, descs_inner
 
 
 # subclass_metas is needed at runtime to compute which indices are symints in
@@ -242,6 +283,7 @@ def runtime_unwrap_tensor_subclasses(
 
         for attr in attrs:
             inner_tensor = getattr(x, attr)
+            # pyrefly: ignore [missing-attribute]
             inner_meta = meta.attrs.get(attr)
             flatten_subclass(inner_tensor, inner_meta, out=out)
 
@@ -288,7 +330,9 @@ def unwrap_tensor_subclasses_with_indices_to_original(wrapped_args):
     ret_unwrapped = []
     ret_indices_to_original = []
     for i, a in enumerate(wrapped_args):
-        a_unwrapped = unwrap_tensor_subclasses([a], append_symints=False)
+        a_unwrapped, _ = unwrap_tensor_subclasses(
+            [a], [DummyAOTInput(9999)], append_symints=False
+        )
         ret_unwrapped.extend(a_unwrapped)
         n = len(a_unwrapped)
         ret_indices_to_original.extend([i] * n)
@@ -305,8 +349,8 @@ def remap_unwrapped_subclass_arg_indices(wrapped_args, static_input_indices):
         if is_traceable_wrapper_subclass(arg):
             num_indices = (
                 len(get_plain_tensors(typing.cast(Tensor, arg), out=[]))
-                + len(filter_symints(arg.size()))
-                + len(filter_symints(arg.stride()))
+                + len(enumerate_filter_symints(arg.size()))
+                + len(enumerate_filter_symints(arg.stride()))
             )
 
         for _ in range(num_indices):
@@ -370,7 +414,7 @@ def wrap_tensor_subclasses(
     # we computed subclass metadata on every forward output, but this did **not** include activations
     # created by the partitioner.
     # as a result, `unwrapped_args` here will correspond to (*unwrapped_user_fw_outs, *activations),
-    # but `subclass_metas` will only correspond to subclass metatadata on `user_fw_outs`.
+    # but `subclass_metas` will only correspond to subclass metadata on `user_fw_outs`.
     # We then need to make sure that we return (*wrapped_user_fw_outs, *activations).
     if num_fw_outs_saved_for_bw is not None:
         assert len(unwrapped_args) == num_args_tallied + num_fw_outs_saved_for_bw, (
@@ -383,9 +427,9 @@ def wrap_tensor_subclasses(
             return wrapped_args + activations
         return tuple(list(wrapped_args) + list(activations))
     else:
-        assert (
-            len(unwrapped_args) == num_args_tallied
-        ), f"Expected {len(unwrapped_args)} == {num_args_tallied}"
+        assert len(unwrapped_args) == num_args_tallied, (
+            f"Expected {len(unwrapped_args)} == {num_args_tallied}"
+        )
         return tuple(wrapped_args)
 
 
@@ -396,7 +440,7 @@ def wrap_tensor_subclasses(
 def wrap_tensor_subclasses_maybe_joint(
     unwrapped_args, *, is_joint_structure: bool, meta: ViewAndMutationMeta
 ) -> Union[tuple[Any, ...], list[Any]]:
-    # Since this function is re-used for both inference and joint graphs,
+    # Since this function is reused for both inference and joint graphs,
     if is_joint_structure:
         assert isinstance(unwrapped_args, tuple) and len(unwrapped_args) == 2
         assert isinstance(unwrapped_args[0], (tuple, list)) and isinstance(
