@@ -21,8 +21,10 @@ from torch._higher_order_ops.flex_attention import (
 from torch._higher_order_ops.foreach_map import _foreach_map, foreach_map
 from torch._higher_order_ops.hints_wrap import hints_wrapper
 from torch._higher_order_ops.invoke_subgraph import invoke_subgraph
+from torch._higher_order_ops.local_map import local_map_hop
 from torch._higher_order_ops.map import map
 from torch._higher_order_ops.out_dtype import out_dtype
+from torch._higher_order_ops.print import print
 from torch._higher_order_ops.run_const_graph import run_const_graph
 from torch._higher_order_ops.scan import scan
 from torch._higher_order_ops.strict_mode import strict_mode
@@ -33,6 +35,7 @@ from torch._higher_order_ops.while_loop import (
 )
 from torch._higher_order_ops.wrap import (
     dynamo_bypassing_wrapper,
+    inductor_compiled_code,
     tag_activation_checkpoint,
     wrap_activation_checkpoint,
     wrap_with_autocast,
@@ -73,4 +76,7 @@ __all__ = [
     "aoti_call_delegate",
     "map",
     "while_loop_stack_output",
+    "local_map_hop",
+    "print",
+    "inductor_compiled_code",
 ]
