@@ -64,9 +64,7 @@ def _is_symmetric_quant(qscheme: "torch.qscheme") -> bool:
 
 
 def _is_float_qparams(qscheme: "torch.qscheme") -> bool:
-    return qscheme in [
-        torch.per_channel_affine_float_qparams,
-    ]
+    return qscheme == torch.per_channel_affine_float_qparams
 
 
 class FakeQuantizeBase(ABC, Module):
@@ -177,9 +175,10 @@ class FakeQuantize(FakeQuantizeBase):
         super().__init__()
         # Populate quant_min/quant_max to observer_kwargs if valid
         if quant_min is not None and quant_max is not None:
-            assert quant_min <= quant_max, (
-                "quant_min must be less than or equal to quant_max"
-            )
+            if quant_min > quant_max:
+                raise AssertionError(
+                    "quant_min must be less than or equal to quant_max"
+                )
             dtype = observer_kwargs.get("dtype", torch.quint8)
             if hasattr(observer, "p"):
                 # In case observer is _PartialWrapper, dtype can be stored in
@@ -187,8 +186,12 @@ class FakeQuantize(FakeQuantizeBase):
                 dtype = getattr(getattr(observer, "p", {}), "keywords", {}).get(
                     "dtype", dtype
                 )
-            assert torch.iinfo(dtype).min <= quant_min, "quant_min out of bound"
-            assert quant_max <= torch.iinfo(dtype).max, "quant_max out of bound"
+            # pyrefly: ignore [bad-argument-type]
+            if torch.iinfo(dtype).min > quant_min:
+                raise AssertionError("quant_min out of bound")
+            # pyrefly: ignore [bad-argument-type]
+            if quant_max > torch.iinfo(dtype).max:
+                raise AssertionError("quant_max out of bound")
             observer_kwargs.update({"quant_min": quant_min, "quant_max": quant_max})
         observer_kwargs["is_dynamic"] = is_dynamic
         self.activation_post_process = observer(**observer_kwargs)
@@ -210,11 +213,12 @@ class FakeQuantize(FakeQuantizeBase):
             if hasattr(self.activation_post_process, "ch_axis")
             else -1
         )
-        assert _is_per_channel(self.qscheme) or _is_per_tensor(self.qscheme), (
-            "Only per channel and per tensor quantization are supported in fake quantize"
-            + " got qscheme: "
-            + str(self.qscheme)
-        )
+        if not (_is_per_channel(self.qscheme) or _is_per_tensor(self.qscheme)):
+            raise AssertionError(
+                "Only per channel and per tensor quantization are supported in fake quantize"
+                + " got qscheme: "
+                + str(self.qscheme)
+            )
         self.is_per_channel = _is_per_channel(self.qscheme)
 
     @torch.jit.export
@@ -295,7 +299,10 @@ class FakeQuantize(FakeQuantizeBase):
                 if name == "scale":
                     self.scale.resize_(val.shape)
                 else:
-                    assert name == "zero_point"
+                    if name != "zero_point":
+                        raise AssertionError(
+                            "Expected 'zero_point' but got different state key"
+                        )
                     self.zero_point.resize_(val.shape)
                 # For torchscript module we need to update the attributes here since we do not
                 # call the `_load_from_state_dict` function defined module.py
@@ -303,7 +310,10 @@ class FakeQuantize(FakeQuantizeBase):
                     if name == "scale":
                         self.scale.copy_(val)
                     else:
-                        assert name == "zero_point"
+                        if name != "zero_point":
+                            raise AssertionError(
+                                "Expected 'zero_point' but got different state key"
+                            )
                         self.zero_point.copy_(val)
             elif strict:
                 missing_keys.append(key)
@@ -329,17 +339,19 @@ class FixedQParamsFakeQuantize(FakeQuantize):
     # TODO: rename observer to observer_ctr
     def __init__(self, observer):
         super().__init__(observer=observer)
-        assert type(self.activation_post_process) == FixedQParamsObserver, (
-            f"{self.__class__.__name__}'s observer must be a {FixedQParamsObserver.__name__}"
-        )
+        if type(self.activation_post_process) is not FixedQParamsObserver:
+            raise AssertionError(
+                f"{self.__class__.__name__}'s observer must be a {FixedQParamsObserver.__name__}"
+            )
         self._observer_ctr = observer
         self.scale = self.activation_post_process.scale
         self.zero_point = self.activation_post_process.zero_point
-        assert _is_per_tensor(self.qscheme), (
-            "Only per tensor quantization is supported"
-            + " FixedQParamsFakeQuantize module, got qscheme:"
-            + str(self.qscheme)
-        )
+        if not _is_per_tensor(self.qscheme):
+            raise AssertionError(
+                "Only per tensor quantization is supported"
+                + " FixedQParamsFakeQuantize module, got qscheme:"
+                + str(self.qscheme)
+            )
 
     @torch.jit.export
     def calculate_qparams(self):  # type: ignore[override]
@@ -382,12 +394,13 @@ class FusedMovingAvgObsFakeQuantize(FakeQuantize):
         **observer_kwargs: Any,
     ) -> None:
         super().__init__(observer, quant_min, quant_max, **observer_kwargs)
-        assert isinstance(
+        if not isinstance(
             self.activation_post_process,
             (MovingAverageMinMaxObserver, MovingAveragePerChannelMinMaxObserver),
-        ), (
-            "Fused observer+fake_quant module only works with MovingAverageMinMaxObserver"
-        )
+        ):
+            raise AssertionError(
+                "Fused observer+fake_quant module only works with MovingAverageMinMaxObserver"
+            )
         self.register_buffer("fake_quant_enabled", torch.tensor([1], dtype=torch.long))
         self.register_buffer("observer_enabled", torch.tensor([1], dtype=torch.long))
         self.is_symmetric_quant = _is_symmetric_quant(
