@@ -4,7 +4,6 @@ import math
 import os
 import weakref
 from functools import lru_cache
-from typing import Optional
 
 import torch
 from torch._dynamo.utils import warn_once
@@ -121,7 +120,7 @@ def slicer(dim, slice_range, *tensors):
 def multidim_slicer(dims, slices, *tensors):
     for t in tensors:
         s = [slice(None)] * t.dim()
-        for d, d_slice in zip(dims, slices):
+        for d, d_slice in zip(dims, slices, strict=False):
             if d is not None:
                 s[d] = d_slice
         yield t[tuple(s)]
@@ -140,7 +139,7 @@ def grid_partitioner(full_grid, grid_blocks, tensor_dims_map):
     import itertools
 
     def generate_grid_points():
-        for fg, mg in zip(full_grid, grid_blocks):
+        for fg, mg in zip(full_grid, grid_blocks, strict=False):
             yield range(0, fg, mg)
 
     def generate_sliced_tensors(slices):
@@ -149,9 +148,10 @@ def grid_partitioner(full_grid, grid_blocks, tensor_dims_map):
 
     for grid_point in itertools.product(*generate_grid_points()):
         grid = [
-            min(fg - gp, mg) for fg, gp, mg in zip(full_grid, grid_point, grid_blocks)
+            min(fg - gp, mg)
+            for fg, gp, mg in zip(full_grid, grid_point, grid_blocks, strict=False)
         ]
-        slices = [slice(gp, gp + g) for gp, g in zip(grid_point, grid)]
+        slices = [slice(gp, gp + g) for gp, g in zip(grid_point, grid, strict=False)]
         # grid_points are iterated in a "contiguous" order, i.e.
         # left dimensions traversed slower than right dimensions.
         # This order is reversed for CUDA grids.
@@ -173,7 +173,8 @@ def launch_kernel(kernel, tensor_dims_map, full_grid, grid_blocks=None):
                 return max(1, min(g, mg))
 
         grid_blocks = tuple(
-            valid_grid_dim(g, mg) for g, mg in zip(grid_blocks, cuda_max_grid)
+            valid_grid_dim(g, mg)
+            for g, mg in zip(grid_blocks, cuda_max_grid, strict=False)
         )  # type: ignore[assignment]
 
     for grid, *sliced_tensors in grid_partitioner(
@@ -385,6 +386,7 @@ def scatter_mm(blocks, others, indices_data, *, accumulators=None):
                 g1 = c_offsets[r + 1]
                 for g in range(g0, g1):
                     p, q = pq[g]
+
                     accumulators[r] += blocks[p] @ others[q]
         else:
             _scatter_mm2(blocks, others, c_offsets, pq, accumulators)
@@ -1120,12 +1122,12 @@ def _int_bsr_dense_addmm(
     *,
     beta=1,
     alpha=1,
-    left_alpha: Optional[torch.Tensor] = None,
-    right_alpha: Optional[torch.Tensor] = None,
-    out: Optional[torch.Tensor] = None,
+    left_alpha: torch.Tensor | None = None,
+    right_alpha: torch.Tensor | None = None,
+    out: torch.Tensor | None = None,
     skip_checks: bool = False,
-    max_grid: Optional[tuple[Optional[int], Optional[int], Optional[int]]] = None,
-    meta: Optional[dict] = None,
+    max_grid: tuple[int | None, int | None, int | None] | None = None,
+    meta: dict | None = None,
 ):
     if out is None and dense.dtype is torch.int8:
         f_name = "_int_bsr_dense_addmm"
@@ -1161,12 +1163,12 @@ def bsr_dense_addmm(
     *,
     beta=1,
     alpha=1,
-    left_alpha: Optional[torch.Tensor] = None,
-    right_alpha: Optional[torch.Tensor] = None,
-    out: Optional[torch.Tensor] = None,
+    left_alpha: torch.Tensor | None = None,
+    right_alpha: torch.Tensor | None = None,
+    out: torch.Tensor | None = None,
     skip_checks: bool = False,
-    max_grid: Optional[tuple[Optional[int], Optional[int], Optional[int]]] = None,
-    meta: Optional[dict] = None,
+    max_grid: tuple[int | None, int | None, int | None] | None = None,
+    meta: dict | None = None,
 ):
     """Compute
 
@@ -1296,19 +1298,31 @@ def bsr_dense_addmm(
     assert alpha != 0
 
     def kernel(grid, *sliced_tensors):
+        # pyrefly: ignore [unsupported-operation]
         _bsr_strided_addmm_kernel[grid](
             *ptr_stride_extractor(*sliced_tensors),
+            # pyrefly: ignore  # bad-argument-count
             beta,
             alpha,
+            # pyrefly: ignore  # bad-keyword-argument, bad-argument-type
             beta_is_one=beta == 1,
+            # pyrefly: ignore  # bad-keyword-argument, bad-argument-type
             beta_is_nonzero=beta != 0,
+            # pyrefly: ignore  # bad-keyword-argument, bad-argument-type
             alpha_is_one=alpha == 1,
+            # pyrefly: ignore  # bad-keyword-argument, bad-argument-type
             left_alpha_is_one=left_alpha_is_one,
+            # pyrefly: ignore  # bad-keyword-argument, bad-argument-type
             right_alpha_is_one=right_alpha_is_one,
+            # pyrefly: ignore  # bad-keyword-argument, bad-argument-type
             BLOCKSIZE_ROW=BM,
+            # pyrefly: ignore  # bad-keyword-argument, bad-argument-type
             BLOCKSIZE_INNER=BK,
+            # pyrefly: ignore  # bad-keyword-argument
             BLOCKSIZE_COL=BN,
+            # pyrefly: ignore  # bad-keyword-argument
             allow_tf32=dot_out_dtype == tl.float32,
+            # pyrefly: ignore  # bad-keyword-argument, bad-argument-type
             acc_dtype=dot_out_dtype,
             **meta,
         )
@@ -1425,6 +1439,7 @@ if has_triton():
 
                 mat1_block = tl.load(
                     mat1_block_ptrs + mat1_col_block_stride * k_offsets[None, :],
+                    # pyrefly: ignore [index-error]
                     mask=mask_k[None, :],
                     other=0.0,
                 )
@@ -1433,6 +1448,7 @@ if has_triton():
                     mat2_block_ptrs
                     + mat2_tiled_col_stride * col_block
                     + mat2_row_block_stride * k_offsets[:, None],
+                    # pyrefly: ignore [index-error]
                     mask=mask_k[:, None],
                     other=0.0,
                 )
@@ -1627,12 +1643,17 @@ if has_triton():
                 beta,
                 is_beta_zero,
                 *blocksize,
+                # pyrefly: ignore  # bad-argument-count
                 k,
                 tile_k,
                 *ptr_stride_extractor(*sliced_tensors),
+                # pyrefly: ignore  # bad-keyword-argument, bad-argument-type
                 acc_dtype=acc_dtype,
+                # pyrefly: ignore  # bad-keyword-argument, bad-argument-type
                 allow_tf32=allow_tf32,
+                # pyrefly: ignore  # unexpected-keyword
                 num_stages=1,
+                # pyrefly: ignore  # unexpected-keyword
                 num_warps=4,
             )
 
@@ -1645,9 +1666,9 @@ if has_triton():
         *,
         beta=1.0,
         alpha=1.0,
-        out: Optional[torch.Tensor] = None,
+        out: torch.Tensor | None = None,
         skip_checks: bool = False,
-        max_grid: Optional[tuple[Optional[int], Optional[int], Optional[int]]] = None,
+        max_grid: tuple[int | None, int | None, int | None] | None = None,
     ):
         f_name = "sampled_addmm"
 
@@ -1729,10 +1750,10 @@ if has_triton():
         bsr: torch.Tensor,
         dense: torch.Tensor,
         *,
-        out: Optional[torch.Tensor] = None,
+        out: torch.Tensor | None = None,
         skip_checks: bool = False,
-        max_grid: Optional[tuple[Optional[int], Optional[int], Optional[int]]] = None,
-        meta: Optional[dict] = None,
+        max_grid: tuple[int | None, int | None, int | None] | None = None,
+        meta: dict | None = None,
     ):
         f_name = "bsr_dense_mm"
         m, _kl = bsr.shape[-2:]
@@ -1917,6 +1938,7 @@ if has_triton():
         def kernel(grid, *sliced_tensors):
             _bsr_softmax_kernel[grid](
                 *ptr_stride_extractor(*sliced_tensors),
+                # pyrefly: ignore  # bad-argument-count
                 row_block,
                 col_block,
                 max_row_nnz,
@@ -1944,10 +1966,10 @@ if has_triton():
         query: torch.Tensor,
         key: torch.Tensor,
         value: torch.Tensor,
-        attn_mask: Optional[torch.Tensor],
+        attn_mask: torch.Tensor | None,
         dropout_p: float = 0.0,
         is_causal: bool = False,
-        scale: Optional[float] = None,
+        scale: float | None = None,
     ):
         f_name = "_scaled_dot_product_attention"
         check(not is_causal, f"{f_name}(): is_causal == True is not supported.")
@@ -1970,6 +1992,7 @@ if has_triton():
         if attn_mask.dtype is not torch.bool:
             check_dtype(f_name, attn_mask, query.dtype)
 
+        # pyrefly: ignore [not-callable]
         sdpa = sampled_addmm(
             attn_mask, query, key.transpose(-2, -1), beta=0.0, skip_checks=False
         )
@@ -1981,8 +2004,10 @@ if has_triton():
             )
         scale_factor = 1 / math.sqrt(query.size(-1)) if scale is None else scale
         sdpa.values().mul_(scale_factor)
+        # pyrefly: ignore [not-callable]
         sdpa = bsr_softmax(sdpa)
         torch.nn.functional.dropout(sdpa.values(), p=dropout_p, inplace=True)
+        # pyrefly: ignore [not-callable]
         sdpa = bsr_dense_mm(sdpa, value)
         return sdpa
 
@@ -2087,8 +2112,11 @@ if has_triton():
         if "allow_tf32" not in meta:
             meta.update(allow_tf32=dot_out_dtype == tl.float32)
         _scatter_mm2_kernel[grid](
+            # pyrefly: ignore  # bad-argument-type
             M,
+            # pyrefly: ignore  # bad-argument-type
             K,
+            # pyrefly: ignore  # bad-argument-type
             N,
             blocks,
             blocks.stride(0),
@@ -2107,7 +2135,9 @@ if has_triton():
             pq_indices,
             pq_indices.stride(0),
             pq_indices.stride(1),
+            # pyrefly: ignore  # bad-argument-type
             dot_out_dtype=dot_out_dtype,
+            # pyrefly: ignore  # bad-argument-type
             **meta,
         )
 
@@ -2290,6 +2320,7 @@ if has_triton():
         _scatter_mm6_kernel[grid](
             B,
             Ms,
+            # pyrefly: ignore  # bad-argument-type
             Ks,
             N,
             blocks,
@@ -2308,6 +2339,7 @@ if has_triton():
             r_offsets,
             p_offsets,
             q_offsets,
+            # pyrefly: ignore  # bad-argument-type
             dot_out_dtype=dot_out_dtype,
             **meta,
         )
