@@ -16,7 +16,7 @@ def has_triton_package() -> bool:
 @functools.cache
 def get_triton_version(fallback: tuple[int, int] = (0, 0)) -> tuple[int, int]:
     try:
-        import triton  # noqa: F401
+        import triton
 
         major, minor = tuple(int(v) for v in triton.__version__.split(".")[:2])
         return (major, minor)
@@ -45,7 +45,12 @@ def has_triton_experimental_host_tma() -> bool:
                     create_2d_tma_descriptor,
                 )
 
-                return True
+                try:
+                    from triton.tools.experimental_descriptor import enable_in_pytorch
+
+                    return enable_in_pytorch()
+                except ImportError:
+                    return True
             except ImportError:
                 pass
 
@@ -105,6 +110,21 @@ def has_triton_tma_device() -> bool:
     return False
 
 
+@functools.cache
+def has_datacenter_blackwell_tma_device() -> bool:
+    import torch
+
+    if (
+        torch.cuda.is_available()
+        and torch.cuda.get_device_capability() >= (10, 0)
+        and torch.cuda.get_device_capability() < (11, 0)
+        and not torch.version.hip
+    ):
+        return has_triton_tma_device() and has_triton_tensor_descriptor_host_tma()
+
+    return False
+
+
 @functools.lru_cache(None)
 def has_triton_stable_tma_api() -> bool:
     if has_triton_package():
@@ -129,6 +149,11 @@ def has_triton() -> bool:
     if not has_triton_package():
         return False
 
+    from torch._inductor.config import triton_disable_device_detection
+
+    if triton_disable_device_detection:
+        return False
+
     from torch._dynamo.device_interface import get_interface_for_device
 
     def cuda_extra_check(device_interface: Any) -> bool:
